package com.diagbot.aggregate;

import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.facade.BillFacade;
import com.diagbot.facade.HighRiskFacade;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.IndicationPushVO;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Description: 提醒类聚合
 * @author: zhoutg
 * @time: 2019/10/16 13:37
 */
@Component
public class IndicationAggregate {

    @Autowired
    BillFacade billFacade;
    @Autowired
    HighRiskFacade highRiskFacade;

    @DataProvider("indicationAll")
    public IndicationDTO indicationAll(
            @InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
            @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO,
            @DataConsumer("bill") IndicationDTO billDTO,
            @DataConsumer("highrisk") IndicationDTO highrisk) {
        IndicationDTO res = new IndicationDTO();
        if (billDTO != null) {
            res.setBillMsgList(billDTO.getBillMsgList());
            res.getDebugStr().addAll(billDTO.getDebugStr()); // 调试信息
        }
        if (highrisk != null) {
            res.setHighRiskList(highrisk.getHighRiskList());
            res.getDebugStr().addAll(highrisk.getDebugStr()); // 调试信息
        }
        return res;
    }

    /**
     * 开单合理性
     *
     * @param wordCrfDTO
     * @param indicationPushVO
     * @return
     */
    @DataProvider("bill")
    public IndicationDTO bill(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
                              @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO) {
        long start = System.currentTimeMillis();
        IndicationDTO res = new IndicationDTO();
        // 开单合理性
        if (indicationPushVO != null && indicationPushVO.getRuleTypeList().contains("2")) {
            billFacade.billFac(indicationPushVO, wordCrfDTO, res);
            CoreUtil.getDebugStr(start, "开单规则", res.getDebugStr());
            return res;
        }
        return null;
    }

    /**
     * 高风险提示
     *
     * @param wordCrfDTO
     * @param indicationPushVO
     * @return
     */
    @DataProvider("highrisk")
    public IndicationDTO highrisk(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
                              @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO) {
        long start = System.currentTimeMillis();
        IndicationDTO res = new IndicationDTO();
        // 高风险提示
        if (indicationPushVO != null && indicationPushVO.getRuleTypeList().contains("3")) {
            highRiskFacade.highRiskFac(indicationPushVO, wordCrfDTO, res);
            CoreUtil.getDebugStr(start, "高风险提示", res.getDebugStr());
            return res;
        }
        return null;
    }
}
