package com.diagbot.repository;

import com.diagbot.dto.*;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class LisNode {

	public LisBillNeoDTO ListoLISDTO(LIS lis) {

		LisBillNeoDTO lisBillNeoDTO = new LisBillNeoDTO();

		Gender gender = lis.getGender();
		if (null!=gender) {
			lisBillNeoDTO.setGender(NeoUtil.updateNodeInfo(gender.getName(), null, null, null));
		}


		Age age = lis.getAge();
		if (null!=age) {
			NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
			ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
			ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
			ageNeoDTO.setName("年龄");
			lisBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
		}

		Set<ClinicalFinding> clinicalfindings = lis.getClinicalfindings();
		for (ClinicalFinding clinicalfinding : clinicalfindings) {
			String name = clinicalfinding.getName();
			Double min = clinicalfinding.getMinval();
			Double max = clinicalfinding.getMaxval();
			String unit = clinicalfinding.getUnit();
			lisBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(name, min, max, unit));
		}

		Set<Vital> vitals = lis.getVitals();
		for (Vital vital : vitals) {
			String name = vital.getName();
			Double min = vital.getMinval();
			Double max = vital.getMaxval();
			String unit = vital.getUnit();
			lisBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(name, min, max, unit));
		}

		Set<Medicine> conflict_medicines = lis.getConflict_medicines();
		for (Medicine conflict_medicine : conflict_medicines) {
			String name = conflict_medicine.getName();
			lisBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}

		Set<AllergicMed> allergicmeds = lis.getAllergicmeds();
		for (AllergicMed allergicmed : allergicmeds) {
			String name = allergicmed.getName();
			lisBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}

		Set<Operation> operations = lis.getOperations();
		for (Operation operation : operations) {
			String name = operation.getName();
			lisBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}

		Set<Procedure> procedures = lis.getProcedures();
		for (Procedure procedure : procedures) {
			String name = procedure.getName();
			lisBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}

		Set<PACS> pacss = lis.getPacs();
		for (PACS pacs : pacss) {
			String name = pacs.getName();
			Double min = pacs.getMinval();
			Double max = pacs.getMaxval();
			String unit = pacs.getUnit();
			lisBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(name, min, max, unit));
		}

		Set<Disease> diseases = lis.getDiseases();
		for (Disease disease : diseases) {
			String name = disease.getName();
			lisBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}

		Set<Group> groups = lis.getGroups();
		for (Group group : groups) {
			String name = group.getName();
			lisBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(name, null, null, null));
		}


		return lisBillNeoDTO;
	}


	/**
	 * 处理化验开单合理性
	 */
	public List<BillNeoDTO> getLisDetailBill(Map<String, String> listerm, LisRepository lisRepository) {
		List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
		BillNeoDTO billNeoDTO;

		LisBillNeoDTO lisBillNeoDTO;

		LIS lis;
		String lisname;

		for (String term : listerm.keySet()) {
			billNeoDTO = new BillNeoDTO();
			lisname = listerm.get(term);
			billNeoDTO.setName(term);
			billNeoDTO.setStandname(lisname);

			lisBillNeoDTO = new LisBillNeoDTO();

			lis = lisRepository.findByNameIs(lisname);

			if (lis != null) {
				lisBillNeoDTO = ListoLISDTO(lis);
			}

			billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

			billNeoDTOs.add(billNeoDTO);
		}

		return billNeoDTOs;
	}



	/**
	 * 获取化验细项名称列表
	 */
	public List<LisDetailDTO> getLisDetailList(String name, LisRepository lisRepository) {
		String nodename;

		List<LisDetailDTO> lislist = new ArrayList<>();
		LisDetailDTO lisDetailDTO;

		List<LIS> lislt = lisRepository.findByNameLike("*"+name+"*");

		for (LIS lis: lislt) {
			lisDetailDTO = new LisDetailDTO();
			nodename = lis.getName();
			lisDetailDTO.setUniqueName(nodename);

			if (lis.getLisPacks().size()>0) {
				List<LISPack> lispacklist = new ArrayList<>(lis.getLisPacks());
				lisDetailDTO.setName(lispacklist.get(0).getName());
			}

			lislist.add(lisDetailDTO);
		}


		return lislist;
	}
}

