package com.diagbot.util;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.model.entity.Clinical;
import org.apache.commons.lang.StringUtils;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: zhoutg
 * @time: 2020/7/30 15:18
 */
public class CoreUtil {

    /**
     * 将列表中属性对应的值以list形式返回
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list, String propertyName) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                String val = (String)getFieldValue(t, propertyName);
                if (StringUtil.isNotBlank(val)) {
                    res.add(val);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    /**
     * 将列表中属性(name)对应的值以list形式返回
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list) {
        return getPropertyList(list, "name");
    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, String propertyName, String standName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, propertyName, standName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, Map<String, String> map) {
        return setPropertyList(list, "name", "standName", map);
    }


    /**
     * 循环向上转型, 获取对象的 DeclaredField
     * @param object : 子类对象
     * @param fieldName : 父类中的属性名
     * @return 父类中的属性对象
     */
    public static Field getDeclaredField(Object object, String fieldName){
        Field field = null ;

        Class<?> clazz = object.getClass() ;

        for(; clazz != Object.class ; clazz = clazz.getSuperclass()) {
            try {
                field = clazz.getDeclaredField(fieldName) ;
                return field ;
            } catch (Exception e) {
                //这里甚么都不要做！并且这里的异常必须这样写，不能抛出去。
                //如果这里的异常打印或者往外抛，则就不会执行clazz = clazz.getSuperclass(),最后就不会进入到父类中了
            }
        }

        return null;
    }

    /**
     * 直接读取对象的属性值, 忽略 private/protected 修饰符, 也不经过 getter
     * @param object : 子类对象
     * @param fieldName : 父类中的属性名
     * @return : 父类中的属性值
     */
    public static Object getFieldValue(Object object, String fieldName){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, fieldName) ;

        //抑制Java对其的检查
        field.setAccessible(true) ;

        try {
            //获取 object 中 field 所代表的属性值
            return field.get(object) ;

        } catch(Exception e) {
            e.printStackTrace() ;
        }

        return null;
    }

    /**
     * 直接设置对象属性值, 忽略 private/protected 修饰符, 也不经过 setter
     * @param object : 子类对象
     * @param name : 父类中的属性名
     * @param standName : 要替换的属性名
     * @param value : 将要设置的值
     */

    public static void setFieldValue(Object object, String name, String standName, Map<String, String> value){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name) ;

        //抑制Java对其的检查
        field.setAccessible(true) ;

        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String)field.get(object);
            if (value != null && value.get(key) != null) {
                Field standField = getDeclaredField(object, standName) ;
                standField.setAccessible(true) ;
                standField.set(object, value.get(key));
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    public static <T> List<String> setPropertyList(List<T> list, String name, String detailName,String uniqueName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, name, detailName,uniqueName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    public static void setFieldValue(Object object, String name, String standName,String unique, Map<String, String> value){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name) ;
        Field field1 = getDeclaredField(object, standName) ;
        //抑制Java对其的检查
        field.setAccessible(true) ;
        field1.setAccessible(true) ;

        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String)field.get(object);
            String key1 = (String)field1.get(object);
            String lis_c = key+key1;
            if (value != null && value.get(lis_c) != null) {
                Field standField = getDeclaredField(object, unique) ;
                standField.setAccessible(true) ;
                standField.set(object, value.get(lis_c));
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    /**
     * 开单合理性通用提示信息
     *
     * @param errMsg 提心信息
     * @param name 项目名称
     * @return
     */
    public static BillMsg getCommonBillMsg(String errMsg, String name,String standName) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format(errMsg + ",不宜开%s", name);
        billMsg.setMsg(msg);
        billMsg.setCompareName(errMsg);
        billMsg.setCompareStandName(standName);
        return billMsg;
    }

    /**
     * 开单合理性通用提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param compareName 匹配名称
     * @param compareStandName 匹配标准名称
     * @return
     */
    public static BillMsg getCommonBillMsg(String orderName, String orderStandName, String compareName, String compareStandName) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format(compareName + ",不宜开%s", orderName);
        billMsg.setMsg(msg);
        billMsg.setCompareName(compareName);
        billMsg.setCompareStandName(compareStandName);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        return billMsg;
    }

    /**
     * 开单合理性通用提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param compareName 匹配名称
     * @param compareStandName 匹配标准名称
     * @param type 类型
     * @return
     */
    public static BillMsg getCommonBillMsg(String orderName, String orderStandName, String compareName,
                                           String compareStandName, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者" + compareName + ",不宜开%s", orderName);
        billMsg.setMsg(msg);
        billMsg.setCompareName(compareName);
        billMsg.setCompareStandName(compareStandName);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 开单合理性固定提示信息
     *
     * @param msg
     * @param msg
     * @return
     */
    public static BillMsg getBillMsgWithConst(String msg) {
        BillMsg billMsg = new BillMsg();
        billMsg.setMsg(msg);
        return billMsg;
    }

    /**
     * 将对象添加到列表中
     *
     * @param list
     * @param t
     * @param <T>
     */
    public static <T> void addBeanToList(List<T> list, T t) {
        if (list == null) {
            list = new ArrayList<T>();
        }
        if (t != null) {
            list.add(t);
        }
    }

    /**
     * 比较字符串是否相等
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static boolean compareStr(NodeNeoDTO nodeNeoDTO, String value) {
        boolean flag = false;
        if (StringUtil.isNotBlank(value)) {
            if (value.equals(nodeNeoDTO.getVal())) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 比较数值大小（int）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static boolean compareNum(NodeNeoDTO nodeNeoDTO, int value) {
        boolean flag = false;
        BigDecimal min = nodeNeoDTO.getMin();
        BigDecimal max = nodeNeoDTO.getMax();

        if (min != null && max != null) {
            int minAge = min.intValue();
            int maxAge = max.intValue();
            if (minAge <= value && value <= maxAge) {
                flag = true;
            }
        } else if (min != null && max == null) {
            int minAge = min.intValue();
            if (minAge <= value) {
                flag = true;
            }
        } else if (min == null && max != null) {
            int maxAge = max.intValue();
            if (value <= maxAge) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 比较数值大小（double）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static boolean compareNum(NodeNeoDTO nodeNeoDTO, double value) {
        boolean flag = false;
        BigDecimal min = nodeNeoDTO.getMin();
        BigDecimal max = nodeNeoDTO.getMax();

        if (min != null && max != null) {
            int minAge = min.intValue();
            int maxAge = max.intValue();
            if (minAge <= value && value <= maxAge) {
                flag = true;
            }
        } else if (min != null && max == null) {
            int minAge = min.intValue();
            if (minAge <= value) {
                flag = true;
            }
        } else if (min == null && max != null) {
            int maxAge = max.intValue();
            if (value <= maxAge) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 获取提示信息（double）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static String getNumMsg(NodeNeoDTO nodeNeoDTO, String name, double value, String unit) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name);
        sb.append(subZeroAndDot(String.valueOf(value))); // 去掉小数点后的0
        if (StringUtil.isBlank(unit)) {
            sb.append(unit);
        }
        sb.append("(禁忌范围[");
        if (nodeNeoDTO.getMin() != null) {
            sb.append(nodeNeoDTO.getMin());
        }
        sb.append(",");
        if (nodeNeoDTO.getMax() != null) {
            sb.append(nodeNeoDTO.getMax());
        }
        // if (nodeNeoDTO.getUnit() != null) {
        //     sb.append(nodeNeoDTO.getUnit());
        // }
        sb.append("])");
        return sb.toString();
    }

    public static String subZeroAndDot(String s){
        if(s.indexOf(".") > 0){
            s = s.replaceAll("0+?$", "");//去掉多余的0
            s = s.replaceAll("[.]$", "");//如最后一位是.则去掉
        }
        return s;
    }

    /**
     * 获取提示信息（例如：年龄19[0,20]）
     *
     * @param nodeNeoDTO
     * @return
     */
    public static String getStrMsgWithRange(NodeNeoDTO nodeNeoDTO, String name, String value) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name).append(value).append("[");
        if (nodeNeoDTO.getMin() != null) {
            sb.append(nodeNeoDTO.getMin());
        }
        sb.append(",");
        if (nodeNeoDTO.getMax() != null) {
            sb.append(nodeNeoDTO.getMax());
        }
        sb.append("]");
        return sb.toString();
    }

    /**
     * 获取提示信息（String）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static String getStrMsg(NodeNeoDTO nodeNeoDTO, String name, String value) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name).append(value);
        return sb.toString();
    }

    /**
     * list 转 string
     * @param list
     * @return
     */
    public static  String listConvertString(List<String> list){
        return StringUtils.join(list, ",");
    }

    /**
     * 输出调试信息
     *
     * @param start
     * @param msg
     * @param debugStr
     */
    public static void getDebugStr(long start, String msg, List<String> debugStr) {
        long end = System.currentTimeMillis();
        debugStr.add(msg + "需要" + (end - start) / 1000.0 + "秒");
    }

    /**
     * 输出调试信息
     *
     * @param start
     * @param msg
     * @param debugStr
     */
    public static void getDebugStr(long start, long end, String msg, List<String> debugStr) {
        debugStr.add(msg + "需要" + (end - start) / 1000.0 + "秒");
    }

    /**
     * 输出调试信息
     *
     * @param start
     * @param msg
     * @param debugStr
     */
    public static void getDebugStrFirst(long start, long end, String msg, List<String> debugStr) {
        debugStr.add(0, msg + "需要" + (end - start) / 1000.0 + "秒");
    }

    public static void main(String[] args) {

        System.out.println(getCommonBillMsg("男性", "尿常规",""));

        List<Clinical> clinicals = new ArrayList<>();
        Clinical c1 = new Clinical();
        c1.setName("a");
        clinicals.add(c1);
        Clinical c2 = new Clinical();
        c2.setName("b");
        clinicals.add(c2);
        Clinical c3 = new Clinical();
        c3.setName("c");
        clinicals.add(c3);

        System.out.println(getPropertyList(clinicals));
        Map<String, String> map = new HashMap<>();
        map.put("a", "A");
        map.put("b", "B");

        setPropertyList(clinicals, map);
    }
}
